/***************************************************************************
 * blitz/alg/balance.cc     
 * function implementation of vectorized Hessengbern-form balance
 *
 * $Id: balance.h,v 1.2 2002/10/26 20:55:49 mdsuresh Exp $
 *
 * Copyright (C) 2002 Suresh kumar Devanathan <mdsuresh@yahoo.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Suggestions:          blitz-dev@oonumerics.org
 * Bugs:                 blitz-bugs@oonumerics.org
 *
 * For more information, please see the Blitz++ Home Page:
 *    http://oonumerics.org/blitz/
 *
 ***************************************************************************
 * $Log: balance.cc,v $
 * Revision 1.0  2001/01/24 20:22:49  mdsuresh
 * Updated copyright date in headers.
 **/

#ifndef _VT_FUNCTIONS_BALANCE_CC
#define _VT_FUNCTIONS_BALANCE_CC

#ifndef _VT_PACK_H
#error the implementation of balance.cc requires inclusion of <blitz/alg/port.h> \
       prior to the inclusion of this file.
#endif      

namespace _VT_Pack {
	namespace Functions {
		namespace	Linear {
			namespace EigenSystems	{

				template<class T>
					void Balance(T& a)
				{
					int j,i;
					int	n =	_VT_rows(a);

					_VT_PRECHECK( _VT_rows(a) == _VT_cols(a),
						"This	operation is only valid	on square matrices.\n"
						"Invalid shape:" << _VT_shape(a) );

					_VT_numtype(T) s,r,g,f,c,sqrdx;
					_VT_DeclareRangeAll( _VT_RangeAll );

					sqrdx= _VT_pow2( _VT_radix(	_VT_numtype(T)() ));
					bool last=false;

					while (!last ) {
						last=true;
						for (i=0;i<n;i++) {
							_VT_Range _i(i);
							c = _VT_sum( _VT_abs(a(_VT_RangeAll,i))) - a(i,i);
							r = _VT_sum( _VT_abs(a(i,_VT_RangeAll))) - a(i,i);

							if	(c && r) {
								g=r/_VT_radix(_VT_numtype(T)());
								f=1.0;
								s=c+r;
								while (c<g)	{
									f *=	_VT_radix(_VT_numtype(T)());
									c *=	sqrdx;
								}
								g=r*_VT_radix(_VT_numtype(T)());
								while (c>g)	{
									f	/= _VT_radix(_VT_numtype(T)());
									c	/= sqrdx;
								}
								if ((c+r)/f	< 0.95*s) {
									last=0;
									g=1.0/f;
									a(i,_VT_RangeAll)	*=g;
									a(_VT_RangeAll,i)   *=f;

								}
							}
						}
					}
				}
		 }
		}
	}
}

#ifdef _VT_MATLAB_INTERFACE

namespace _VT_MATLAB_NAMESPACE	{
	/* balance: MATLAB equivalent name */
	template<class T>
		T	balance(T &	_arr) {
			T	arr	= _VT_copy(_arr);
			_VT_Pack::Functions::Linear::EigenSystems::Balance(arr);
			return arr;		   
		}
}

#endif // _VT_MATLAB_INTERFACE


#endif // _VT_FUNCTIONS_BALANCE_CC
