/***************************************************************************
* blitz/alg/jacobi.cc     
* function implementation of vectorized jacobi transformation
*
* $Id: jacobi.cc,v 1.2 2002/10/26 09:55:49 mdsuresh Exp $
*
* Copyright (C) 2002 Suresh kumar Devanathan <mdsuresh@yahoo.com>
*
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* Suggestions:          blitz-dev@oonumerics.org
* Bugs:                 blitz-bugs@oonumerics.org
*
* For more information, please see the Blitz++ Home Page:
*    http://oonumerics.org/blitz/
*
***************************************************************************
* $Log: jacobi.cc,v $
* Revision 1.0  2002/10/26 09:22:49  mdsuresh
* Updated copyright date in headers.
**/

#ifndef _VT_FUNCTION_JACOBI_CC
#define _VT_FUNCTION_JACOBI_CC

#ifndef _VT_PACK_H
#error the implementation of Jacobi transformation requires inclusion of <blitz/alg/port.h> \
	prior to the inclusion of this file.
#endif   

namespace _VT_Pack {
	namespace Functions {
		namespace Matrix {
			namespace EigenSystems {


				template<class T>
					void JacobiRotate(T a1, T a2,
					_VT_numtype(T) s,
					_VT_numtype(T) tau
					)
				{
					_VT_Array1D( _VT_numtype(T) ) wk( a1.shape());

					wk = _VT_copy(a1);
					//g=a(i,j);h=a(k,l);
					a1=a1-s*(a2+a1*tau);
					a2=a2+s*(wk-a2*tau);
				}

				template<class T1, class T2>
					void Jacobi(T1& a,T2& d, T1& v, int& nrot)
				{
					int j,iq,ip,i,n;
					n = _VT_rows(a);
					_VT_numtype(T1) tresh,theta,tau,t,sm,s,h,g,c;
					T2 b(n),z(n);
					_VT_resize(d,n);
					_VT_resize(v,n,n);

					_VT_PRECHECK( _VT_rows(a) == _VT_cols(a),
						"This operation is only valid on square matrices.\n"
						"Invalid shape:" << _VT_shape(a) );

					v = 0;
					for(ip = 0; ip < n; ip++) {
						v(ip,ip) = 1.0;
						d(ip) =a(ip,ip);
					}
					/*
					for (ip=1;ip<=n;ip++) {
					for (iq=1;iq<=n;iq++) v[ip][iq]=0.0;
					v[ip][ip]=1.0;
					}*/
					z = 0;           	
					b = d;
					nrot=0;

					for (i=0;i<50;i++) {
						sm=0.0;
						for (ip=0;ip<n-1;ip++) {
							// for (iq=ip+1;iq<n;iq++)
							sm +=  //(ip+1 < n-1) ?
								_VT_sum(_VT_abs(a(ip,_VT_Range(ip+1, n-1))))
								//: 0
                                 ;
						}
						if (sm == 0.0) 
							return;

						if (i < 3)
							tresh=0.2*sm/(n*n);
						else
							tresh=0.0;

						for (ip=0;ip<n-1;ip++) {
							for (iq=ip+1;iq<n;iq++) {
								g=100.0*_VT_abs(a(ip,iq));
								if (i > 4 && _VT_numtype(T1)(_VT_abs(d(ip))+g) == 
									_VT_numtype(T1)(_VT_abs(d(ip)))
									&& _VT_numtype(T1)(_VT_abs(d(iq))+g) ==
									_VT_numtype(T1)(_VT_abs(d(iq))) )
									a(ip,iq)=0.0;
								else if (_VT_abs(a(ip,iq)) > tresh) {
									h=d(iq)-d(ip);
									if (_VT_numtype(T1)(_VT_abs(h)+g) == 
										_VT_numtype(T1)(_VT_abs(h)))
										t=(a(ip,iq))/h;
									else {
										theta=0.5*h/(a(ip,iq));
										t=1.0/(_VT_abs(theta)+_VT_sqrt(1.0+theta*theta));
										if (theta < 0.0) t = -t;
									}
									c=1.0/_VT_sqrt(1+t*t);
									s=t*c;
									tau=s/(1.0+c);
									h=t*a(ip,iq);
									z(ip) -= h;
									z(iq) += h;
									d(ip) -= h;
									d(iq) += h;
									a(ip,iq)=0.0;
									{
										_VT_Range j(0,ip-1);
										JacobiRotate(a(j,ip),a(j,iq),s,tau);
									}
									{
										_VT_Range j(ip+1,iq-1);
										JacobiRotate(a(ip,j),a(j,iq),s,tau);
									}
									{
										_VT_Range j(iq+1,n-1); 
											JacobiRotate(a(ip,j),a(iq,j),s,tau);
									}
									{
									_VT_Range j(0,n-1);
										JacobiRotate(v(j,ip),v(j,iq),s,tau);
									}
									++(nrot);
								}
							}
						}
						b += z;
						d = b;
						z = 0;
						/*
						for (ip=1;ip<=n;ip++) {
						b[ip] += z[ip];
						d[ip]=b[ip];
						z[ip]=0.0;
						}
						*/
					}
					_VT_CONVERGE_FAIL(false,"Too many iterations in routine jacobi");
				}

			}
		}
	}
}


#ifdef _VT_MATLAB_INTERFACE

namespace _VT_MATLAB_NAMESPACE {
   using namespace _VT_Pack;
	/* Numerical Recipies compatible names. */
	/* Since there are no MATLAB equivalents. */


	template<class T>
		_VT_rhs_< _VT_Array1D( _VT_numtype(T) ),
		_VT_Array2D( _VT_numtype(T) ) >  jacobi(T _arr)
	{
		T arr = _VT_copy(_arr);
		_VT_Array1D( _VT_numtype(T) ) d;
		_VT_Array2D( _VT_numtype(T) ) v;
		int _nrot;
		_VT_Pack::Functions::Matrix::EigenSystems::Jacobi(arr, d, v , _nrot);

		return _VT_rhs(d , v);
	}

}    


#endif // _VT_MATLAB_INTERFACE

#endif // _VT_FUNCTION_JACOBI_CC
