/***************************************************************************
 * blitz/alg/lu.cc     
 * function implementation of vectorized LU decomposition, inversion 
 * and backsubstitution
 *
 * $Id: lu.cc,v 1.2 2002/10/26 20:55:49 mdsuresh Exp $
 *
 * Copyright (C) 2002 Suresh kumar Devanathan <mdsuresh@yahoo.com>
 * 
 * $Id linalg.h Version 1.1
 * Copyright (C) 1998-1999 Brent Worden WordenWare <Brent@Worden.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Suggestions:          blitz-dev@oonumerics.org
 * Bugs:                 blitz-bugs@oonumerics.org
 *
 * For more information, please see the Blitz++ Home Page:
 *    http://oonumerics.org/blitz/
 *
 ***************************************************************************
 * $Log: lu.cc,v $
 * Revision 1.0  2002/10/26 20:22:49  mdsuresh
 * Updated copyright date in headers.
 **/

#ifndef _VT_FUNCTION_LU_CC
#define _VT_FUNCTION_LU_CC

#ifndef _VT_PACK_H
#error the implementation of LU facorization requires inclusion of <blitz/alg/port.h> \
	prior to the inclusion of this file.
#endif


namespace _VT_Pack {
	namespace  Functions {
		namespace Matrix {
			namespace Factorization {

				template<class T>
					void LUInverse(T& a)
				{
					int i, j;
					int n = _VT_rows(a);

					_VT_PRECHECK( _VT_rows(a) == _VT_cols(a),
						"This operation is only valid on square matrices.\n"
						"Invalid shape:" << _VT_shape(a) );

					T  ai(n, n);
					ai = 0;
					_VT_Array1D( int ) indx(n);
					indx = 0;
					_VT_Array1D( _VT_numtype(T) ) col(n);
					int d;
					_VT_DeclareRangeAll( _VT_RangeAll);
					//transposeSelf(1,0);

					LUDecomposition(a, indx, d);

					for(j = 0; j < n; j++){
						col = 0;
						col(j) = _VT_numtype(T)(1);
						LUBackSubstitution(a, indx, col);
						ai( _VT_RangeAll,j) =  col;
						//for(i = 0; i < n; i++){
						//	ai(i,j) = col(i);
						//}
					}
					a = ai;
					return;

				}

				template<class T1, class T2, class T3>
					void LUBackSubstitution(T1& a, T2& indx, T3& b)
					//-------------------------------------------------------------------
					// Given a matrix a and permutation vector indx returned from ludcmp,
					// this routines solves the set of linear equations a.x = b.  On 
					// input b holds the right-hand side vector.  On output, it holds the
					// solution vector x.  a and indx are not modified by this routine
					// and can be left in place for successive colls with different
					// right-hand sides b.
					//-------------------------------------------------------------------
				{
					int i, ii, ip, j;
					_VT_numtype(T3) _sum;
					int n = _VT_rows(a);

					_VT_PRECHECK( _VT_rows(a) == _VT_cols(a),
						"This operation is only valid on square matrices.\n"
						"Invalid shape:" << _VT_shape(a) );
					_VT_PRECHECK( _VT_rows(a) == _VT_rows(indx),
						"This operation requires intermediate value indx to have"
						"same length as indx.\n" 
						"a:"    << _VT_shape(a) << "\n" <<
						"indx:" << _VT_shape(indx) );

					_VT_PRECHECK( _VT_rows(a) == _VT_rows(b),
						"This operation requires intermediate value indx to have"
						"same length as indx.\n" 
						"a:" << _VT_shape(a) << "\n" <<
						"b:" << _VT_shape(b) );

					ii = -1;

					for(i = 0; i < n; i++){
						ip = indx(i);
						_sum = b(ip);
						b(ip) = b(i);
						if(ii > -1){
							_VT_Range j(ii,i-1);
							_sum -= _VT_sum(a(i,j) * b(j));

						} else if(_sum){
							ii = i;
						}
						b(i) = _sum;
					}
					for(i = n - 1; i >= 0; i--){
						_VT_Range j(i+1,n-1);
						b(i) = ( b(i) - (
							//(i+1 < n) ? 
							(_VT_sum(a(i,j) * b(j)))
							//:0
							) ) / a(i,i);

					}
					return;

				}

				template<class T>
					void LUDecomposition(T& a)
				{
					_VT_Array1D(int)  indx(_VT_rows(a));
					indx = 0;
					int d;
				    LUDecomposition(a,indx,d);
				}

				template<class T1, class T2>
					void LUDecomposition(T1& a, T2& indx, int& d)
					//-------------------------------------------------------------------
					// See above comment.  d is output as 1 or -1 depending on whether
					// the number of row interchanges was even or odd, respectively.
					// This routine is used in combination with lubksb to solve linear
					// equations or invert a matrix.
					//-------------------------------------------------------------------
				{
					int i, imax, j, k;
					_VT_numtype(T1) big, dum, temp,_sum;
					int n = _VT_rows(a);

					_VT_PRECHECK( _VT_rows(a) == _VT_cols(a),
						"This operation is only valid on square matrices.\n"
						"Invalid shape:" << _VT_shape(a) );

					_VT_Array1D( _VT_numtype(T1) ) vv(n);
					//_VT_numtype(T1) zero(0);
					//_VT_numtype(T1) one(1);

					d = 1;
					_VT_DeclareRangeAll( _VT_RangeAll);

					cout << a << "\n";

					vv =  _VT_max( _VT_abs( a(_VT_tensor::i, _VT_tensor::j)), _VT_tensor::j);

					cout << vv << "\n";


					_VT_ILLCONDITION( !_VT_any( vv == 0.0), 
						"This matrix has an inconditional property." );

					vv  = _VT_pow(vv,-1);



					for(j = 0; j < n; j++){
						_VT_Range q(j,n-1);  
						imax = j + _VT_maxIndex_1( vv(q) * _VT_abs( a(q , j) ) , 0 );

						if( j!=imax) {
							_VT_swap( a(imax, _VT_RangeAll), a(j, _VT_RangeAll));
							d = -d;
							vv(imax) = vv(j);
						}
						indx(j) = imax;

						if( a(j,j) == 0.0) a(j,j) = _VT_tiny( _VT_numtype(T1)() );

					    q = _VT_Range(j+1,n-1);
						a(q,j) = a(q,j) / a(j,j);
						a(q,q) = a(q,q) - (a(q,j)(_VT_tensor::i) * a(j,q)(_VT_tensor::j));

					}

					return;
				}
			}
		}
	} 
}

#ifdef _VT_MATLAB_INTERFACE

namespace _VT_MATLAB_NAMESPACE {

	/* Numerical Recipies compatible name. */

	template<class T>
		T luinv(T _input)
	{
		T input = _VT_copy(_input);
		_VT_Pack::Functions::Matrix::Factorization::LUInverse(input);
		return input;
	}


#ifdef _VT_INVERSION_LU
	template<class T>
		T inv(T input) 
	{
		return luinv(input);
	}
#endif


	/* lu: MATLAB   */    
	/* TODO:Needs work  */ 
	template<class T>
		T lu(T _input)
	{
		T input = _VT_copy(_input);
		_VT_Pack::Functions::Matrix::Factorization::LUDecomposition(input);
		return input;
	}



}

#endif // _VT_MATLAB_INTERFACE

#endif // _VT_FUNCTION_LU_CC
