/***************************************************************************
* blitz/alg/port.h     
* this file implements several classes/function that will 
* ease the implementation of several of these algorithms to different
* librarys like POOMA/ MTL etc
*
* Porting should be really really easy 
*
* $Id: port.h,v 1.2 2002/10/26 20:55:49 mdsuresh Exp $
*
* Copyright (C) 2002 Suresh kumar Devanathan <mdsuresh@yahoo.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* Suggestions:          blitz-dev@oonumerics.org
* Bugs:                 blitz-bugs@oonumerics.org
*
* For more information, please see the Blitz++ Home Page:
*    http://oonumerics.org/blitz/
*
***************************************************************************
* $Log: port.h,v $
* Revision 1.0  2001/01/24 20:22:49  mdsuresh
* Updated copyright date in headers.
**/


#ifndef _VT_PACK_H
#define _VT_PACK_H

#include <blitz/array.h>
#include <blitz/lhsrhs.h>


#define _VT_BASE_NAMESPACE   blitz
#define _VT_MATLAB_NAMESPACE   blitz
#define _VT_MATLAB_INTERFACE
#define _VT_Pack blitz

#define _VT_typename         _bz_typename
#define _VT_numtype(T)       _VT_typename T::T_numtype
#define _VT_Array            _VT_BASE_NAMESPACE::Array
#define _VT_abs              _VT_BASE_NAMESPACE::abs
#define _VT_any              _VT_BASE_NAMESPACE::any
#define _VT_sum              _VT_BASE_NAMESPACE::sum
#define _VT_sqrt             _VT_BASE_NAMESPACE::sqrt
#define _VT_atan             _VT_BASE_NAMESPACE::atan
#define _VT_log              _VT_BASE_NAMESPACE::log
#define _VT_exp              _VT_BASE_NAMESPACE::exp
#define _VT_floor            _VT_BASE_NAMESPACE::floor
#define _VT_product          _VT_BASE_NAMESPACE::product
#define _VT_max              _VT_BASE_NAMESPACE::max
#define _VT_min              _VT_BASE_NAMESPACE::min
#define _VT_maxIndex         _VT_BASE_NAMESPACE::maxIndex
#define _VT_minIndex         _VT_BASE_NAMESPACE::minIndex
#define _VT_firstIndex       _VT_BASE_NAMESPACE::firstIndex


#define _VT_pow              _VT_BASE_NAMESPACE::pow
#define _VT_pow2             _VT_BASE_NAMESPACE::pow2
#define _VT_Range            _VT_BASE_NAMESPACE::Range


#define _VT_TinyVector       _VT_BASE_NAMESPACE::TinyVector
#define _VT_Index1D          _VT_BASE_NAMESPACE::TinyVector<int,1>
#define _VT_Index2D          _VT_BASE_NAMESPACE::TinyVector<int,2>
#define _VT_Index3D          _VT_BASE_NAMESPACE::TinyVector<int,3>
#define _VT_ref              _VT_BASE_NAMESPACE::ref
#define _VT_lhs              _VT_BASE_NAMESPACE::lhs
#define _VT_where            _VT_BASE_NAMESPACE::where
#define _VT_rhs              _VT_BASE_NAMESPACE::rhs
//#define _VT_swap             _VT_BASE_NAMESPACE::swap
#define _VT_cycleArrays      _VT_BASE_NAMESPACE::cycleArrays

#define _VT_lhs_             _VT_BASE_NAMESPACE::_lhs
#define _VT_rhs_             _VT_BASE_NAMESPACE::_rhs

#define _VT_PRECHECK         BZPRECHECK
#define _VT_PRECONDITION     BZPRECONDITION
#define _VT_CONVERGE_FAIL   BZPRECHECK
#define _VT_ILLCONDITION     BZPRECHECK

#define _VT_tensor           _VT_BASE_NAMESPACE::tensor
#define _VT_Array1D(T)       _VT_BASE_NAMESPACE::Array<T,1>
#define _VT_Array2D(T)       _VT_BASE_NAMESPACE::Array<T,2>
#define _VT_Array3D(T)       _VT_BASE_NAMESPACE::Array<T,3>
#define _VT_radix            _VT_BASE_NAMESPACE::radix
#define _VT_tiny             _VT_BASE_NAMESPACE::tiny

#define _VT_ManualVectorize1D     


//#define _VT_firstIndex_1(T1, T2)   _VT_BASE_NAMESPACE::firstIndex(T1)(T2)
//#define _VT_maxIndex_1(T1, T2)     _VT_BASE_NAMESPACE::maxIndex(T1)(T2)
#define _VT_DeclareRangeAll(T)      _VT_BASE_NAMESPACE::Range T


namespace _VT_Pack {

	template<class T>
	int _VT_maxIndex_1(T a,int i)
	{
		return  _VT_BASE_NAMESPACE::maxIndex(a)(i);
	}

	
	template<class T, int n>
    void _VT_swap(_VT_Array<T,n> a,_VT_Array<T,n> b)
    {
        _VT_Array<T,n> temp = _VT_copy(a);
        a = b;
        b = temp;
        //_VT_BASE_NAMESPACE::swap(a,b);
    }

	template<class T>
	int _VT_firstIndex_1(T a,int _i)
	{
		for(int i=0; i < _VT_rows(a); i++)
			 if( a(i) )		
				   return i;
				
		return _VT_tiny(int());
	}

/*

	template<class T>
	int _VT_cols( Array<T,1> & a)
	{ 
	   return a.rows();
	}
	

	
	template<class T>
	int _VT_cols( Array<T,2> & a)
	{ 
	   return a.cols();
	}
	
	template<class T>
	int _VT_rows( Array<T, 1>&  a)
	{ 
	   return 1 ;
	}
	
	template<class T>
	int _VT_rows( Array<T,2> & a)
	{ 
	   return a.rows();
	}


*/

	template<class T>  
		int _VT_rows(const T & a) 
	{
		return a.rows();
	}    

	template<class T>  
		int _VT_cols(const T&  a) 
	{
		return a.cols();
	}   

	template<class T>
		int _VT_size(const T&  a, int n)
	{
		return a.extent(n);
	}




/*	template<class T>
		bool _VT_any(T a)
	{
		return _any(a);
	}
*/
	template<class T>
		void _VT_resize(T & a, int n)
	{
		a.resize(n);
	}

	template<class T>
		void _VT_resize(T & a, int n1, int n2)
	{
		return a.resize(n1, n2);
	}

	template<class T>
		int _VT_rank(T  a)
	{
		return a.rank();
	}

	template<class T, int n>
		_VT_TinyVector<int, n> _VT_shape(_VT_Array<T,n>& a)
	{
		return a.shape();
	}
	/*
	template<class T>
	    T _VT_max(T a, T b)
	{
	      return _VT_BASE_NAMESPACE::max(a,b);
	}
	
	template<class T>
	    T _VT_max(T a, T b)
	{
	      return _VT_BASE_NAMESPACE::min(a,b);
	}
	*/



	template<class T>
		T _VT_copy(const T& a)
	{
		return a.copy();
	}

	template<class T>
	   _VT_numtype(T) *  _VT_dataFirst( T& a)
	{
		return a.dataFirst();
	}

	template<class T>
		T _VT_sign(T u,T v) 

	{
		return (v)>=0.0 ? _VT_abs(u) : -_VT_abs(u); 
	}



}
/*
#define _VT_rows         _VT_Pack::rows
#define _VT_cols         _VT_Pack::cols
#define _VT_resize       _VT_Pack::resize
#define _VT_rank         _VT_Pack::rank
#define _VT_size         _VT_Pack::size
#define _VT_copy         _VT_Pack::copy
#define _VT_shape        _VT_Pack::shape
#define _VT_any          _VT_Pack::any
#define _VT_dataFirst    _VT_Pack::dataFirst
#define _VT_sign         _VT_Pack::sign
*/

#endif


